/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Author Quan Sun (quan.sun.nz@gmail.com)
 *    Copyright (C) 2013 University of Waikato, Hamilton, New Zealand
 *
 */

package artforests.app;

import artforests.core.ARTForests;
import artforests.core.MultiTargetInstance;
import artforests.core.Tools;
import org.w3c.dom.Element;
import weka.core.Instance;
import weka.core.Instances;

public class BuildModel {

    public static void main(String[] args) throws Exception {
        
        Misc.printAppHeader();
        
        String xmlPath = "C:\\Users\\Quan\\Desktop\\Dropbox\\ResearchProjects\\artforests\\data\\trainConfig.xml";
        
        if (args.length != 0) {
            xmlPath = args[0];           
        }
        
        Element eElement = XMLReader.readXMLConfigFile(xmlPath);

        String modelPath = eElement.getElementsByTagName("modelPath").item(0).getTextContent();
        String trainFeaturesPath = eElement.getElementsByTagName("trainFeaturesPath").item(0).getTextContent();
        String trainRanksPath = eElement.getElementsByTagName("trainRanksPath").item(0).getTextContent();
        
        //
        int numTrees = Integer.parseInt(eElement.getElementsByTagName("numTrees").item(0).getTextContent());
        int numRandFeatures = Integer.parseInt(eElement.getElementsByTagName("numRandFeatures").item(0).getTextContent());
        int numMinInstances = Integer.parseInt(eElement.getElementsByTagName("numMinInstances").item(0).getTextContent());
         
        Instances featureData = Tools.loadCSVInstances(trainFeaturesPath);
        Instances rankData = Tools.loadCSVInstances(trainRanksPath);

        Instances data = new Instances(featureData, 0);
        for (int i = 0; i < featureData.numInstances(); i++) {
            Instance featureInst = featureData.instance(i);
            Instance rankInst = rankData.instance(i);
            MultiTargetInstance mInst = new MultiTargetInstance(featureInst);
            double[] ranking = new double[rankData.numAttributes()];
            for (int r = 0; r < rankData.numAttributes(); r++) {
                ranking[r] = rankInst.value(r);
            }
            mInst.setRanking(ranking);
            data.add(mInst);
        }

        System.out.println("Num. Training Instances: " + data.numInstances());
        System.out.println("Num. Targets: " + rankData.numAttributes());
        System.out.println("Num. (ART) trees to built: " + numTrees);
        System.out.println("numRandFeatures: " + numRandFeatures + " (0 means log2(numRandFeatures)+1)");
        System.out.println("numMinInstances: " + numMinInstances);
        
        System.out.println("Building ART Forests");
        
        ARTForests ranker = new ARTForests();
        ranker.setNumIterations(numTrees);
        ranker.setK(numRandFeatures);
        ranker.setNumMinInstances(numMinInstances);
        ranker.buildRanker(data);
        
        weka.core.SerializationHelper.write(modelPath, ranker);     
        System.out.println("Model saved to: " + modelPath);
        System.out.println("Done!");
        System.out.println();
    }
}
