/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Author Quan Sun (quan.sun.nz@gmail.com)
 *    Copyright (C) 2013 University of Waikato, Hamilton, New Zealand
 *
 */

package artforests.app;

import artforests.core.ARTForests;
import artforests.core.EvalTool;
import artforests.core.MultiTargetInstance;
import artforests.core.Tools;
import org.w3c.dom.Element;
import weka.core.Instance;
import weka.core.Instances;

public class Predict {

    public static void main(String[] args) throws Exception {
        
        Misc.printAppHeader();
        
        String xmlPath = "C:\\Users\\Quan\\Desktop\\Dropbox\\ResearchProjects\\artforests\\data\\testConfig.xml";
        
        if (args.length != 0) {
            xmlPath = args[0];        
        }
        
        Element eElement = XMLReader.readXMLConfigFile(xmlPath);

        String modelPath = eElement.getElementsByTagName("modelPath").item(0).getTextContent();
        String testFeaturesPath = eElement.getElementsByTagName("testFeaturesPath").item(0).getTextContent();
        String testRanksPath = eElement.getElementsByTagName("testRanksPath").item(0).getTextContent();       
        String predRanksPath = eElement.getElementsByTagName("predRanksPath").item(0).getTextContent();

        Instances featureData = Tools.loadCSVInstances(testFeaturesPath);
        Instances rankData = Tools.loadCSVInstances(testRanksPath);

        Instances data = new Instances(featureData, 0);
        for (int i = 0; i < featureData.numInstances(); i++) {
            Instance featureInst = featureData.instance(i);
            Instance rankInst = rankData.instance(i);
            MultiTargetInstance mInst = new MultiTargetInstance(featureInst);
            double[] ranking = new double[rankData.numAttributes()];
            for (int r = 0; r < rankData.numAttributes(); r++) {
                ranking[r] = rankInst.value(r);
            }
            mInst.setRanking(ranking);
            data.add(mInst);
        }
        
        System.out.println("Num. Test Instances: " + data.numInstances());
        System.out.println("Num. Targets: " + rankData.numAttributes());
        
        StringBuilder sb = new StringBuilder();
        String header = "";
        for (int i = 0; i < rankData.numAttributes(); i++) {
            header += "RankOfItem-" + (i + 1) + ",";
        }
        header = header.substring(0, header.length() - 1);
        sb.append(header).append(Tools.newLine());

        ARTForests ranker = (ARTForests) weka.core.SerializationHelper.read(modelPath);
        //
        double avgCC = 0;

        for (int i = 0; i < data.numInstances(); i++) {
            MultiTargetInstance inst = (MultiTargetInstance) data.instance(i);
            double[] predRanking = ranker.recommendRanking(inst);
            double[] targetRanking = inst.getRanking();

            String line = "";
            for (int j = 0; j < predRanking.length; j++) {
                line += predRanking[j] + ",";
            }
            line = line.substring(0, line.length() - 1);
            sb.append(line).append(Tools.newLine());

            avgCC += EvalTool.computeSpearmanCC(predRanking, targetRanking);
        }
        avgCC /= data.numInstances();
        System.out.println("Average test set spearmans correlation: " + avgCC);
        Tools.saveStringBuilder(sb, predRanksPath);
        System.out.println("Prediction file saved to: " + predRanksPath);
    }
}
